% Compute intercell fluxes according to the Richtmyer method, or two step Lax-Wendroff method
%   ____  _      _     _                             
%  |  _ \(_) ___| |__ | |_ _ __ ___  _   _  ___ _ __ 
%  | |_) | |/ __| '_ \| __| '_ ` _ \| | | |/ _ \ '__|
%  |  _ <| | (__| | | | |_| | | | | | |_| |  __/ |   
%  |_| \_\_|\___|_| |_|\__|_| |_| |_|\__, |\___|_|
%                                    |___/           
%
%  Name of program: Richtmyer.m
%
%  Developed by: Enrico Bertolazzi
%                Department of Mechanical and Structural Engineering
%                enrico.bertolazzi@ing.unitn.it
%                http://www.ing.unitn.it/~bertolaz/
%
%  Last revision: 3 June 2008
%
%
%  Purpose: Compute the the Lax-Friedrichs numerical flux
%
%            / DX           \    1  /            DT                    \
%  Richtmyer | ----, QL, QR | = --- | QR + QL - ---- ( F(QR) - F(QL) ) |
%            \ DT           /    2  \            DX                    /
%
%  The fuction numFlux(DX/DT,QL,QR) compute the intercell flux between two cell of size DX
%  with left state QL and right stagte QR. Different numerical fluxes results in different
%  numerical schemes. Some other functions are used to complete the definition of the numerical
%  scheme.
%
%  The scheme can be visualized in the following way
% 
%   +===+  The cell where the average value Qi is "stored".
%
%   +...+  Ghost cell. These cells are used to extrapolate values for the
%          construction of the bounary conditions. Two ghost cell for baoundary
%          are enough for all the considered fluxes.
%
%     +    The nodal values of the boundary of the cells.
%
%   +....+....+====+====+====+====+====    =====+======+......+.....+
%   : Q  : Q  | Q  | Q  | Q  |    |         Q   | Q    | Q    : Q   :
%   :  -1:  0 |  1 |  2 |  3 |               N-1|  N   |  N+1 :  N+2:
%   +....+....+====+====+====+====+====    =====+======+......+.....+
%
%  MATLAB cannot use negative index so that index are shifted by 2.
%
%  INPUTS:
%
%     DXDT = The ratio DX/DT: the grid speed.
%
%     Flux = the function handle of the routine comuting the flux F(Q).
%            Using the command FQ = feval( Flux, Q ) then
%            FQ(:,i) contains Flux( Q(:,i) ) 
% 
%     Q    = a matrix m x (N+4) where m is the dimension of the problem
%            and N is the number of the cells.
%
%  OUTPUTS:
%
%     F   = a matrix m n (N+1) with the numerical cells boundary fluxes
%
%     F(:,k) contains the boundary flux between cells k-1 and k.
%
%  See Eq. 5.79, Chapter 5, Ref. 1 of
%  E. F. Toro, "Riemann Solvers and Numerical Methods for Fluid Dynamics"
%  Springer-Verlag, 1997, Second Edition, 1999

function F = Richtmyer( DXDT, Flux, Q )
  QL = Q(:,2:end-2) ;
  QR = Q(:,3:end-1) ;
  %%
  FL = feval( Flux, QL) ;
  FR = feval( Flux, QR) ;
  QS = 0.5 * ( QL + QR + (FL-FR) / DXDT ) ;
  F  = feval( Flux, QS ) ;
end

