%  Given average states compute WENO limited slopes
%  Name of program: WENOSlopeLimiter.m
%
%  Developed by: Enrico Bertolazzi
%                Department of Mechanical and Structural Engineering
%                enrico.bertolazzi@ing.unitn.it
%                http://www.ing.unitn.it/~bertolaz/
%
%  This code is inspired on the library
%    NUMERICA
%    A Library of Source Codes for Teaching,
%    Research and Applications,
%    by E. F. Toro
%    Published by NUMERITEK LTD,
%    Website: www.numeritek.coM
%
%  The algorithm are described in the book:
%
%    E. F. Toro, "Riemann Solvers and Numerical Methods for Fluid Dynamics"
%                 Springer-Verlag, 1997, Second Edition, 1999
%
%  Last revision: 3 June 2008
%
%  The scheme can be visualized in the following way
% 
%   +===+  The cell where the average value Qi is "stored".
%
%   +...+  Ghost cell. These cells are used to extrapolate values for the
%          construction of the bounary conditions. Two ghost cell for baoundary
%          are enough for all the considered fluxes.
%
%   Averages states
%
%   +...+...+===+===+===+===+===    ===+===+...+...+
%    Q1  Q2  Q3                                 QN+4
%
%   Limited slopes
%
%   +...+===+===+===+===+===+===    ===+===+===+...+
%         S1  S2  S3                        SN+2
%
%  INPUTS:
%
%     Q = average values
%
%  OUTPUTS:
%
%     SLOPE = limited slopes
%
function SLOPE = WENOSlopeLimiter( Q )

  % compute slopes
  DQ = Q(2:end) - Q(1:end-1) ;
  
  % left slopes
  LSLOPE = DQ(1:end-1);

  % right slopes
  RSLOPE = DQ(2:end);

  % find minimum slopes
  IL = find( abs(LSLOPE) <  abs(RSLOPE) ) ;
  IR = find( abs(LSLOPE) >= abs(RSLOPE) ) ;

  SLOPE(IL) = LSLOPE(IL) ;
  SLOPE(IR) = RSLOPE(IR) ;

end
